@@ -26,6 +26,8 @@ from gi.repository import GLib
 from pyanaconda import constants
 from pyanaconda.threads import threadMgr, AnacondaThread
 from pyanaconda.ui.gui import GUIObject
+from pyanaconda.ui.gui.utils import escape_markup
+from pyanaconda.i18n import _
 from pyanaconda import nm
 
 __all__ = ["ISCSIDialog"]
@@ -38,7 +40,7 @@ Credentials = namedtuple("Credentials", ["style",
                                          "targetIP", "initiator", "username",
                                          "password", "rUsername", "rPassword"])
 
-NodeStoreRow = namedtuple("NodeStoreRow", ["selected", "notLoggedIn", "name", "iface"])
+NodeStoreRow = namedtuple("NodeStoreRow", ["selected", "notLoggedIn", "name", "iface", "portal"])
 
 def discover_no_credentials(builder):
     return Credentials(STYLE_NONE,
@@ -141,6 +143,7 @@ class ISCSIDialog(GUIObject):
         self._initiatorEntry = self.builder.get_object("initiatorEntry")
 
         self._store = self.builder.get_object("nodeStore")
+        self._storeFilter = self.builder.get_object("nodeStoreFiltered")
 
     def refresh(self):
         self._bindCheckbox.set_active(bool(self.iscsi.ifaces))
@@ -151,7 +154,7 @@ class ISCSIDialog(GUIObject):
 
         self._loginAuthTypeCombo.set_active(0)
 
-        self.builder.get_object("nodeStoreFiltered").set_visible_column(1)
+        self._storeFilter.set_visible_column(1)
 
         self._initiatorEntry.set_text(self.iscsi.initiator)
         self._initiatorEntry.set_sensitive(not self.iscsi.initiatorSet)
@@ -231,7 +234,6 @@ class ISCSIDialog(GUIObject):
             # that subscreen.
             self._add_nodes(self._discoveredNodes)
             self._iscsiNotebook.set_current_page(1)
-            self._okButton.set_sensitive(True)
 
             # If some form of login credentials were used for discovery,
             # default to using the same for login.
@@ -264,9 +266,15 @@ class ISCSIDialog(GUIObject):
         # Now get the node discovery credentials.
         credentials = discoverMap[self._authNotebook.get_current_page()](self.builder)
 
+        discoveredLabelText = _("The following nodes were discovered using the iSCSI initiator "\
+                                "<b>%(initiatorName)s</b> using the target IP address "\
+                                "<b>%(targetAddress)s</b>.  Please select which nodes you "\
+                                "wish to log into:") % \
+                                {"initiatorName": escape_markup(credentials.initiator),
+                                 "targetAddress": escape_markup(credentials.targetIP)}
+
         discoveredLabel = self.builder.get_object("discoveredLabel")
-        discoveredLabel.set_markup(discoveredLabel.get_label() % {"initiatorName": credentials.initiator,
-                                                                  "targetAddress": credentials.targetIP})
+        discoveredLabel.set_markup(discoveredLabelText)
 
         bind = self._bindCheckbox.get_active()
 
@@ -294,7 +302,8 @@ class ISCSIDialog(GUIObject):
         text = widget.get_text()
 
         stripped = text.strip()
-        return "." in stripped and ":" in stripped
+        #iSCSI Naming Standards: RFC 3720 and RFC 3721
+        return "." in stripped
 
     def on_discover_field_changed(self, *args):
         # Make up a credentials object so we can test if it's valid.
@@ -309,7 +318,8 @@ class ISCSIDialog(GUIObject):
     def _add_nodes(self, nodes):
         for node in nodes:
             iface =  self.iscsi.ifaces.get(node.iface, node.iface)
-            self._store.append([False, True, node.name, iface])
+            portal = "%s:%s" % (node.address, node.port)
+            self._store.append([False, True, node.name, iface, portal])
 
         # We should select the first node by default.
         self._store[0][0] = True
@@ -326,7 +336,8 @@ class ISCSIDialog(GUIObject):
             return
 
         # Then, go back and mark just this row as selected.
-        itr = self._store.get_iter(path)
+        itr = self._storeFilter.get_iter(path)
+        itr = self._storeFilter.convert_iter_to_child_iter(itr)
         self._store[itr][0] = not self._store[itr][0]
 
     def _login(self, credentials):
@@ -337,7 +348,8 @@ class ISCSIDialog(GUIObject):
                 continue
 
             for node in self._discoveredNodes:
-                if obj.notLoggedIn and node.name == obj.name:
+                if obj.notLoggedIn and node.name == obj.name \
+                   and obj.portal == "%s:%s" % (node.address, node.port):
                     # when binding interfaces match also interface
                     if self.iscsi.ifaces and \
                        obj.iface != self.iscsi.ifaces[node.iface]:
